# Author: Ketevi A. Assamagan
# BNL, June 12, 2004

# Event selector
include( "AthenaPoolCnvSvc/ReadAthenaPool_jobOptions.py" )
EventSelector = Service( "EventSelector" )
#EventSelector.BackNavigation = True

# Particle Properties
include( "PartPropSvc/PartPropSvc.py" )

# the POOL converters
include( "ParticleEventAthenaPool/AOD_PoolCnv_jobOptions.py")

# Read in the AOD from POOL - AOD.pool.root
# this file is produced by running the AOD builders of the RDO raw data
# The AOD input file
#EventSelector.InputCollections = [ 
#  "AOD.pool.root"
#                                 ]

include( "Htautau_11.0.41.py" )

# The user analysis algorithm to be executed
theApp.Dlls   += [ "TruthParticleAlgs" ]
theApp.Dlls   += [ "AnalysisTools" ]
theApp.Dlls   += [ "UserAnalysis" ]

include ("CBNT_Athena/CBNT_AthenaAware_jobOptions.py")
include ("CBNT_Athena/CBNT_EventInfoAA_jobOptions.py")

CBNT_AthenaAware = Algorithm( "CBNT_AthenaAware" )
CBNT_AthenaAware.Members += [ "AnalysisSkeleton" ]

AnalysisSkeleton = Algorithm( "AnalysisSkeleton" )

# The user analysis cuts - default values to changed by the user as needed

AnalysisSkeleton.ElectronContainer = "ElectronCollection"
AnalysisSkeleton.MCParticleContainer = "SpclMC"

# matching to MC truth cuts
AnalysisSkeleton.DeltaRMatchCut = 0.2
AnalysisSkeleton.MaxDeltaR = 0.9999

# electrons
AnalysisSkeleton.ElectronEtCut  = 10.0*GeV
AnalysisSkeleton.ElectronEtaCut = 2.5
AnalysisSkeleton.ElectronCone   = 0.9

# message level for AnalysisSkeleton
AnalysisSkeleton.OutputLevel = INFO

##########################################
# setup TTree registration svc
# save ROOT histograms and Tuple
THistSvc = Service ( "THistSvc" )
THistSvc.Output = ["AANT DATAFILE='AnalysisSkeleton.aan.root' OPT='UPDATE'"]
theApp.TopAlg += [ "AANTupleStream" ]
AANTupleStream = Algorithm( "AANTupleStream" )
AANTupleStream.ExtraRefNames = [ "StreamESD","Stream1" ]
AANTupleStream.OutputName = 'AnalysisSkeleton.aan.root'
AANTupleStream.WriteInputDataHeader = True
AANTupleStream.OutputLevel = WARNING

# Set output level threshold (2=DEBUG, 3=INFO, 4=WARNING, 5=ERROR, 6=FATAL )
MessageSvc = Service( "MessageSvc" )
MessageSvc.OutputLevel = ERROR

# Number of Events to process
theApp.EvtMax = 500

###################### For interactive analysis
#include ("PyAnalysisCore/InitPyAnalysisCore.py")

###################### Detail time measurement and auditors
# Use auditors
theApp.AuditAlgorithms=True
theApp.Dlls += [ "GaudiAud" ]

include ("LSFTimeKeeper/LSFTimeKeeperOptions.py")
LSFTimeKeeperSvc= Service("LSFTimeKeeperSvc")
LSFTimeKeeperSvc.OutputLevel=DEBUG

theAuditorSvc = AuditorSvc()
theAuditorSvc.Auditors  += [ "ChronoAuditor"]

AthenaPoolCnvSvc = Service("AthenaPoolCnvSvc")
AthenaPoolCnvSvc.UseDetailChronoStat = TRUE

#StoreGateSvc = Service( "StoreGateSvc" )
#StoreGateSvc.Dump = True  #true will dump data store contents
#MessageSvc.OutputLevel = DEBUG

