#ifndef PixelGeometryManager_H
#define PixelGeometryManager_H

#include <string>
#include <iostream>
#include <map>
#include "GeoModelKernel/GeoMaterial.h"
#include "CLHEP/Geometry/Vector3D.h"

class StoredMaterialManager;
class StoreGateSvc;

namespace InDetDD {
  class PixelDetectorManager;
  class SiCommonItems;
}

class PixelGeometryManager {


 public:

  
  PixelGeometryManager();
  virtual ~PixelGeometryManager();
  static PixelGeometryManager* GetPointer(); 

  //
  // GET THE OTHER MANAGERS FROM STOREGATE
  // -------------------------------------

  // Get the material manager:
  virtual StoredMaterialManager* getMaterialManager()=0;

  // PixelDetectorManager
  virtual InDetDD::PixelDetectorManager *GetPixelDDManager()=0;

  // 
  // VERSION TAG
  // -----------
  virtual std::string versionTag() const = 0;

  //
  // DESIGN AND ELEMENT NAMES
  // ------------------------
  virtual void SetDetElementName(std::string)=0;
  virtual std::string GetDetElementName()=0;
  virtual void SetDesignName(std::string)=0;
  virtual std::string GetDesignName()=0;

  virtual void SetBarrelModuleName(std::string)=0;
  virtual std::string GetBarrelModuleName()=0;
  virtual void SetEndcapModuleName(std::string)=0;
  virtual std::string GetEndcapModuleName()=0;
  virtual void SetLayer0ModuleName(std::string)=0;
  virtual std::string GetLayer0ModuleName()=0;

  //
  // BUILDING DEFINITIONS
  // --------------------

  // Do I want the services?
  virtual void SetServices(bool isservice)=0;
  virtual bool DoServices()=0;

  //JBdV Do I want the services? (Omega + Al tube + cables T0 + pigtails + connectors)
  virtual void SetServicesOnLadder(bool isservice)=0;
  virtual bool DoServicesOnLadder()=0;


  // Make elements compatible with G3 digits
  virtual void SetG3CompatibleDigits(bool flag)=0;
  virtual bool G3CompatibleDigits() const=0;

  // Initial layout (2nd layer missing)
  virtual void SetInitialLayout(bool flag)=0;
  virtual bool InitialLayout() const=0;

  // DC1 Geometry. 300 um long pixels and 200 um thick sensor in B layer. 
  virtual void SetDC1Geometry(bool flag)=0;
  virtual bool DC1Geometry() const=0;

  // Control whether callbacks get registered
  virtual void SetAlignable(bool flag)=0;
  virtual bool Alignable() const=0;

  //
  // BUILDER HELPERS
  // ----------------

  virtual void SetEta(int eta)=0;
  virtual void SetPhi(int phi)=0;
  virtual int Eta()=0;
  virtual int Phi()=0;

  // What am I building?
  virtual bool isBarrel()=0;
  virtual bool isEndcap()=0;
  virtual void SetBarrel()=0;
  virtual void SetEndcap()=0;

  // The layer/disk barrel/endcap can be changed by these function.
  virtual void SetCurrentLD(int i)=0;
  virtual int GetLD()=0;

  // Which layers/disks are present?
  virtual bool isLDPresent()=0;

  // The side
  virtual void SetPos()=0;
  virtual void SetNeg()=0;
  virtual int GetSide()=0;

  // The TMT and the Omega
  virtual void SetCurrentTrap(int)=0;
  virtual int GetTrap()=0;
  virtual void SetCurrentParaEl(int i)=0;
  virtual int GetParaEl()=0 ;
  virtual void SetCurrentParaElType(std::string type)=0;
  virtual std::string GetParaElType()=0;
  virtual void SetCurrentTrapType(std::string type)=0;
  virtual std::string GetTrapType()=0;
  virtual void SetCurrentCyliEl(int i)=0;
  virtual int GetCyliEl()=0;
  // the weights (for checking...)
  virtual void AddNStaves(unsigned int i)=0;
  virtual unsigned int NStave()=0;
  virtual void AddWeight(double p)=0;
  virtual double LePoids()=0;
  virtual void AddWeightOmega(double p)=0;
  virtual double WeiOmega()=0;
  virtual void AddWeightTrapTmt(double p)=0;
  virtual double WeiTrapTmt()=0;
  virtual void AddWeightModule(double p)=0;
  virtual double WeiModule()=0;
  virtual void AddWeightCable(double p)=0;
  virtual double WeiCable()=0;
  virtual void AddWeightFluid(double p)=0;
  virtual double WeiFluid()=0;  

  //
  // DETECTOR PARAMTERS
  // ------------------


  // Version Number, for the Barrel/EndCap
  virtual inline int PixelBarrelMajorVersion()=0;
  virtual inline int PixelBarrelMinorVersion()=0;
  virtual inline int PixelEndcapMajorVersion()=0;
  virtual inline int PixelEndcapMinorVersion()=0;  

  // Si Board
  virtual double PixelBoardWidth()=0;
  virtual double PixelBoardLength()=0;
  virtual double PixelBoardThickness()=0;
  virtual double PixelBoardActiveLen()=0;

  // Hybrid
  virtual double PixelHybridWidth()=0;
  virtual double PixelHybridLength()=0;
  virtual double PixelHybridThickness()=0;

  //  Fe Chips (PBRN)
  virtual double PixelChipWidth()=0;
  virtual double PixelChipLength()=0;
  virtual double PixelChipGap()=0;
  virtual double PixelChipThickness()=0;

  // Disk Carbon Structure
  virtual double PixelECCarbonRMin(std::string)=0;
  virtual double PixelECCarbonRMax(std::string)=0;
  virtual double PixelECCarbonThickness(std::string)=0;
  virtual std::string PixelECCarbonMaterial(std::string)=0;

  // Services
  virtual double* PixelServiceR(std::string, int )=0;
  virtual double* PixelServiceZ(std::string, int )=0;
  virtual std::string PixelServiceMaterial(std::string, int )=0;
  virtual int PixelServiceLD(std::string, int )=0;
  virtual int PixelServiceNFrame(std::string)=0;

  //  Atlas Global volume (from ATLS)
  virtual inline double GetATLSRadius()=0;
  virtual inline double GetATLSRmin()=0;
  virtual inline double GetATLSLength()=0;

  //  Pixel container (from PXBG)
  virtual inline double PixelRMin()=0;
  virtual inline double PixelRMax()=0;
  virtual inline double PixelHalfLength()=0;
  virtual inline int PixelBarrelNLayer()=0;

  // Pixel Barrel  (from PXBO)
  virtual inline double PixelBarrelRMin()=0;
  virtual inline double PixelBarrelRMax()=0;
  virtual inline double PixelBarrelHalfLength()=0;
  
  // Pixel Layers Geomtry (PXBI)
  virtual inline double PixelLayerRadius()=0;
  virtual inline double PixelLadderHalfLength()=0;
  virtual inline double PixelLadderWidth()=0;
  virtual double PixelLadderThickness()=0;
  virtual inline double PixelLadderTilt()=0;
  virtual inline int NPixelSectors()=0;
  virtual inline double PixelBalcony()=0;
  virtual inline int PixelNModule()=0;
  virtual inline double PixelModuleDrDistance()=0;
  virtual inline double PixelModuleAngle()=0;
  virtual inline double PixelModulePosition(int)=0;
  virtual inline double PixelModuleShiftFlag(int)=0;
  virtual inline double PixelModuleAngleSign(int)=0;
  virtual inline double PixelGapDetLad()=0;


  // Barrel LAYER CABLES (PBAC)
  virtual inline double PixelCableWidth()=0;
  virtual inline double PixelCableThickness()=0;
  virtual inline double PixelCableZMax()=0;
  virtual inline double PixelCableZMin()=0;
  virtual inline double PixelCableDeltaZ()=0;

  // trapezoid parameters
  virtual inline int PixelNTrap()=0;
  virtual inline int PixelNTrapC()=0;
  virtual inline int PixelNTrapE()=0;
  virtual inline double PixelTrapWidth()=0;
  virtual inline double PixelTrapDzdr()=0;
  virtual inline double PixelTrapHalfLength()=0;
  virtual inline double PixelTrapHalfWidthY()=0;
  virtual inline double PixelTrapHalfWidthXzn()=0;
  virtual inline double PixelTrapHalfWidthXzp()=0;
  virtual inline double PixelTrapPosx()=0;
  virtual inline double PixelTrapPosy()=0;
  virtual inline double PixelTrapPosz()=0;
  virtual inline int PixelTrapLabel()=0;
  virtual inline int PixelTrapMat()=0;

  // Omega parameters
  virtual inline int PixelOmegaNParaEl()=0;
  virtual inline int PixelOmegaNCyliEl()=0;
  virtual inline double PixelPosiHalfLength()=0;
  virtual inline double PixelPosiWidth()=0;
  virtual inline double PixelPosiThick()=0;
  virtual inline double PixelPosiTilt()=0;
  virtual inline double PixelPosiPosx()=0;
  virtual inline double PixelPosiPosy()=0;
  virtual inline double PixelPosiPosz()=0;
  virtual inline double PixelPosiRotz()=0;
  virtual inline int PixelPosiLabel()=0;
  virtual inline double PixelPotiHalfLength(std::string)=0;
  virtual inline double PixelPotiRmin(std::string)=0;
  virtual inline double PixelPotiRmax(std::string)=0;
  virtual inline double PixelPotiPhi1(std::string)=0;
  virtual inline double PixelPotiPhi2(std::string)=0;
  virtual inline double PixelPotiPosx(std::string)=0;
  virtual inline double PixelPotiPosy(std::string)=0;
  virtual inline double PixelPotiPosz(std::string)=0;
  virtual inline int PixelPotiLabel(std::string)=0;
  virtual inline int PixelConnectorANParaEl()=0;
  virtual inline int PixelConnectorCNParaEl()=0;
  virtual inline int PixelConnectorBNCyliEl()=0;
  virtual inline int PixelMaterialPara()=0;
  virtual inline int PixelMaterialCyli(std::string)=0;
  virtual std::string PixelGetMat(int)=0;

  //Fluid parameter
  virtual inline int PixelFluidOrient()=0;


  // Pixel Endcap PXEG
  virtual inline int PixelEndcapNDisk()=0;

  // Pixel Endcap Container PEVO
  virtual inline double PixelEndcapRMin()=0;
  virtual inline double PixelEndcapRMax()=0;
  virtual inline double PixelEndcapZMin()=0;
  virtual inline double PixelEndcapZMax()=0;
  virtual inline int PixelEndcapNSupportFrames()=0;

  // Pixel Disks PXEI
  virtual inline double PixelDiskPosition()=0;
  virtual inline double PixelECSiDz1()=0;
  virtual inline double PixelECSiDz2()=0;
  virtual inline double PixelDiskRMin()=0;
  virtual inline int PixelECNSectors1()=0;
  virtual inline int PixelECNSectors2()=0;

  // Endcap CABLES (PEAC)
  virtual inline double PixelECCablesRMin()=0;
  virtual inline double PixelECCablesRMax()=0;
  virtual double PixelECCablesThickness()=0;
  virtual inline double PixelECCablesDistance()=0;

  // Design Parameter
  //
  virtual inline double DesignRPActiveArea()=0;
  virtual inline double DesignZActiveArea()=0;

  virtual inline int DesignCircuitsPerColumn()=0;
  virtual inline int DesignCircuitsPerRow()=0;

  virtual int DesignCellColumnsPerCircuit(bool isBLayer)=0;
  virtual int DesignCellRowsPerCircuit(bool isBLayer)=0;
  virtual int DesignDiodeColumnsPerCircuit(bool isBLayer)=0;
  virtual int DesignDiodeRowsPerCircuit(bool isBLayer)=0;

  virtual inline double DesignPitchRP(bool isBLayer)=0;
  virtual inline double DesignGapRP()=0;
  virtual inline double DesignPitchZ(bool isBLayer)=0;
  virtual inline double DesignGapZ()=0;

  // Ganged Pixels
  virtual int NumberOfEmptyRows()=0;
  virtual int EmptyRows(int index)=0;
  virtual int EmptyRowConnections(int index)=0;

  // 
  // conditions stuff (probably it shouldnt be here)
  virtual double Voltage(bool isBLayer)=0;
  virtual double Temperature(bool isBLayer)=0;

  // Magnetic field
  virtual const HepVector3D & magneticField(bool isBLayer = false) const=0;
  virtual void setMagneticField(const HepVector3D & field)=0;
  virtual bool useMagneticFieldSvc() const=0;
  virtual void setUseMagneticFieldSvc(bool flag)=0;  

  // CommonItems for Det Elements
  virtual InDetDD::SiCommonItems * commonItems() const=0;
  virtual void setCommonItems(InDetDD::SiCommonItems * commonItems)=0; 


 private:
  
  static PixelGeometryManager *s_geometry_manager;


};
#endif
