#ifndef PixelSwitches_H
#define PixelSwitches_H

// Class to hold the various switches and building options.

class PixelSwitches 
{

public:
  PixelSwitches();

  // Do I want the services?
  void setServices(bool flag);
  bool services() const;

  // JBdV Do I want the services?
  void setServicesOnLadder(bool flag);
  bool servicesOnLadder() const;

  // Make elements compatible with G3 digits
  void setG3CompatibleDigits(bool flag);
  bool g3CompatibleDigits() const;
  
  // Initial layout (2nd layer missing)
  void setInitialLayout(bool flag);
  bool initialLayout() const;
  
  // DC1 Geometry. 300 um long pixels and 200 um thick sensor in B layer. 
  void setDC1Geometry(bool flag);
  bool dc1Geometry() const;
  
  // Control whether callbacks get registered
  void setAlignable(bool flag);
  bool alignable() const;

  // Use magnetic filed service to get B field for Lorentz shift
  void setUseMagneticFieldSvc(bool flag);  
  bool useMagneticFieldSvc() const;

private:

  // flag to build also the services
  bool m_services;

  // JBdV flag to build also the servicesOnLadder
  bool m_servicesOnLadder;

  // flag to make elements compatible with G3 digits
  bool m_g3CompatibleDigits;

  // flag to indicate initial layout (2nd layer and disk missing)
  bool m_initialLayout;
  
  // flag to force B-layer to have 300um pixels and be 200um thick
  bool m_dc1Geometry;

  // control whether callbacks get registered 
  bool m_alignable;

  // control whether to use magnetic field service.
  bool m_useMagFieldSvc;

};

#endif // PixelSwitches_H


