#include "PixelGeoModel/GeoPixelBcmWall.h"
#include "PixelGeoModel/GeoPixelLayer.h"
#include "PixelGeoModel/GeoPixelServices.h"
#include "GeoModelKernel/GeoTube.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoAlignableTransform.h"
#include "InDetReadoutGeometry/PixelDetectorManager.h"
#include "Identifier/Identifier.h"

// For the Msg Svc
#include "AthenaKernel/getMessageSvc.h"
#include "GaudiKernel/MsgStream.h"


using namespace std;

GeoVPhysVol* GeoPixelBcmWall::Build( ) {

  double G10Thick=0.4;
  double CuThick=0.035;
  double wallThick=G10Thick + CuThick;
  double halfY=1;
  double halfZ=1;
 
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");

  const GeoBox* bcmWallBox = new GeoBox(wallThick/2,halfY,halfZ);
  const GeoLogVol* bcmWallLog=new GeoLogVol("bcmWallLog",bcmWallBox,air);
  GeoPhysVol* bcmWallPhys = new GeoPhysVol(bcmWallLog);

return bcmWallPhys;

}



GeoVPhysVol* GeoPixelBcmWall::Build(double halfY, double halfZ, double G10Thick, double CuThick ) {

  MsgStream log(Athena::getMessageSvc(), "PixelDetectorFactory");

  //
  // create the Barrel Mother volume
  // 
//  double rmin = gmt_mgr->PixelBarrelRMin();
//  double rmax = gmt_mgr->PixelBarrelRMax();
//  double halflength = gmt_mgr->PixelBarrelHalfLength();

//  double r=5.639;
//  double z=183.8 + 0.185;

  //double G10Thick=0.4;
  //double CuThick=0.035;
  double wallThick=G10Thick + CuThick;

 
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");

  const GeoBox* bcmWallBox = new GeoBox(wallThick/2,halfY,halfZ);
  const GeoLogVol* bcmWallLog=new GeoLogVol("bcmWallLog",bcmWallBox,air);
  GeoPhysVol* bcmWallPhys = new GeoPhysVol(bcmWallLog);



		//add copper
		GeoMaterial* copper = mat_mgr->getMaterial("std::Copper");

		//bottom layer

		const GeoBox* bcmWallCuBox = new GeoBox(CuThick/2,halfY,halfZ);
		const GeoLogVol* bcmWallCuLog = new GeoLogVol("bcmWallCuLog",bcmWallCuBox,copper);
		GeoPhysVol* bcmWallCuPhys = new GeoPhysVol(bcmWallCuLog);
		
		double x,y,z;
		x = -(wallThick/2 - CuThick/2);
		y = 0;
		z = 0;

		Hep3Vector WallCuPos(x,y,z);
		HepRotation rm;
		GeoTransform* xform = new GeoTransform(HepTransform3D(rm,WallCuPos));
		//GeoNameTag* tag = new GeoNameTag("BCM Module Top Wall");
		//bcmModPhys->add(tag);
		bcmWallPhys->add(xform);
		bcmWallPhys->add(bcmWallCuPhys);


		//add G10

		GeoMaterial* g10 = mat_mgr->getMaterial("std::G10");

		const GeoBox* bcmWallG10Box = new GeoBox(G10Thick/2,halfY,halfZ);
		const GeoLogVol* bcmWallG10Log = new GeoLogVol("bcmWallG10Log",bcmWallG10Box,g10);
		GeoPhysVol* bcmWallG10Phys = new GeoPhysVol(bcmWallG10Log);
		
		//double x,y,z;
		x = (wallThick/2 - G10Thick/2);
		y = 0;
		z = 0;

		Hep3Vector WallG10Pos(x,y,z);
		//HepRotation rm;
		xform = new GeoTransform(HepTransform3D(rm,WallG10Pos));
		//GeoNameTag* tag = new GeoNameTag("BCM Module Top Wall");
		//bcmModPhys->add(tag);
		bcmWallPhys->add(xform);
		bcmWallPhys->add(bcmWallG10Phys);


  return bcmWallPhys;
}
