#include "PixelGeoModel/GeoPixelCable.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"

#include <sstream>

GeoVPhysVol* GeoPixelCable::Build() {
  //
  // Dimensions
  //
  double length = this->Length();
  double thickness = this->Thickness();
  double width = gmt_mgr->PixelCableWidth();
  GeoMaterial* cable = mat_mgr->getMaterial("pix::Cable");
  const GeoBox* cableBox = new GeoBox(thickness/2.,width/2.,length/2.);
  std::string logName = "cableLog";
  std::ostringstream o;
  o << m_moduleNumber;
  logName = logName+o.str();

  GeoLogVol* theCable = new GeoLogVol(logName,cableBox,cable);
  GeoPhysVol* cablePhys = new GeoPhysVol(theCable);
  return cablePhys;
}


double GeoPixelCable::Length() {
  //
  // The cable is running from the beginning of the module to which
  // it is attached to the end of the ladder,
  // zmax gives the end of the ladder and zmin is the begining of the module
  // in absolute value, so the cable lenght is the difference b/w the two
  // the TubeCables factory will have to position this cable in the correct
  // side.
  //
  double zmax = gmt_mgr->PixelCableZMax();
  double zmin = gmt_mgr->PixelCableZMin()+m_moduleNumber*gmt_mgr->PixelCableDeltaZ();
  return zmax-zmin-m_epsilon;
}

double GeoPixelCable::Thickness() {
  //
  // This is calculated from the rad length of the cables and their mass
  // I have to go back on this later when I'll have defined a material
  // manager, for the time being I get the thickness by atlsim, using dtree
  // anf hardwire the numbers in the code...
  // I have to come back on the thickness using the cables recipes
  //
  if(m_moduleNumber == 0) return 2.*0.0028412*cm;
  if(m_moduleNumber == 1) return 2.*0.0056795*cm;
  if(m_moduleNumber == 2) return 2.*0.0056830*cm;
  if(m_moduleNumber == 3) return 2.*0.0056763*cm;
  if(m_moduleNumber == 4) return 2.*0.0056752*cm;
  if(m_moduleNumber == 5) return 2.*0.0057058*cm;
  if(m_moduleNumber == 6) return 2.*0.0056818*cm;

  return 0.;

}
