#include "PixelGeoModel/GeoPixelDiskSupports.h"
#include "GeoModelKernel/GeoTube.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoTransform.h"

#include <sstream>

GeoPixelDiskSupports::GeoPixelDiskSupports() {
  //
  // Initialize the vectors
  //
  //
  // zpos is not needed, but it's here just in case...
  //
  std::string ele[3] = {"Inner", "Central", "Outer"};
  for (int ii =0; ii<3; ii++) {
    double rmin = gmt_mgr->PixelECCarbonRMin(ele[ii]);
    double rmax = gmt_mgr->PixelECCarbonRMax(ele[ii]);
    double halflength = gmt_mgr->PixelECCarbonThickness(ele[ii])/2.;
    double zpos = 0.;
    std::string mat = gmt_mgr->PixelECCarbonMaterial(ele[ii]);
    _rmin.push_back(rmin);
    _rmax.push_back(rmax);
    _zpos.push_back(zpos);
    _halflength.push_back(halflength);
    material.push_back(mat);
  }
}


 
GeoVPhysVol* GeoPixelDiskSupports::Build( ) {
  GeoMaterial* supportMat = mat_mgr->getMaterial(material[_nframe]);
  //
  // Dimensions: (_nframe set with teh SetCylinder method)
  //
  double rmin = _rmin[_nframe];
  double rmax = _rmax[_nframe];
  double halflength = _halflength[_nframe];
  const GeoTube* supportTube = new GeoTube(rmin,rmax,halflength);
  std::string logName = "diskSupLog";
  std::ostringstream o;
  o << _nframe;
  logName = logName+o.str();
  GeoLogVol* theSupport = new GeoLogVol(logName,supportTube,supportMat);
  GeoPhysVol* supportPhys = new GeoPhysVol(theSupport);
  return supportPhys;
}

