#include "PixelGeoModel/GeoPixelLadder.h"
#include "PixelGeoModel/GeoPixelTrapTmt.h"
#include "PixelGeoModel/GeoPixelPosi.h"
#include "PixelGeoModel/GeoPixelModule.h"
#include "PixelGeoModel/GeoPixelSiCrystal.h"
#include "InDetReadoutGeometry/PixelDetectorManager.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoTubs.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"

#include "GeoModelKernel/GeoTransform.h"
#include "GeoModelKernel/GeoAlignableTransform.h"


using std::max;

GeoPixelLadder::GeoPixelLadder(GeoPixelSiCrystal& theSensor) :
  m_theSensor(theSensor)
{
  //
  // Define the log volume in the constructor, so I do it only once.
  //

  //   
  // Length of the ladder is in the db
  //
  double halflength = gmt_mgr->PixelLadderHalfLength();
  //
  // The width is the maximum among the componenst widths
  //
  double width = max( max(
			  gmt_mgr->PixelBoardWidth(),
			  gmt_mgr->PixelHybridWidth()),
		      gmt_mgr->PixelChipWidth());
  //
  // The thickness has to be calculated
  //
  double thickness = this->Thickness();
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  const GeoBox* ladderBox = new GeoBox(thickness/2.,width/2.,halflength);
  theLadder = new GeoLogVol("ladderLog",ladderBox,air);
}
GeoVPhysVol* GeoPixelLadder::Build( ) {
  GeoPhysVol* ladderPhys = new GeoPhysVol(theLadder);
  //
  // Place the Modules
  //
  GeoPixelModule pm(m_theSensor);
  int HalfNModule = gmt_mgr->PixelNModule()/2;
  for(int ii = 0; ii < gmt_mgr->PixelNModule(); ii++) {
    //
    // Which of the two sides is it?
    //
    float side = 1.;
    int jj = ii-HalfNModule;
    gmt_mgr->SetEta(jj);
    if(jj < 0) {
      side = -1.;
      jj = -jj;
    }
//
// Shift down the central module, as its flag (from NOVA) is -1.
//
    float xpos = gmt_mgr->PixelModuleDrDistance()*gmt_mgr->PixelModuleShiftFlag(jj)/2.;
//
// Get the z position from the db
//
    float zpos = gmt_mgr->PixelModulePosition(jj)*side;
    Hep3Vector modulepos(xpos,0.,zpos);
//
//
//
    HepRotation rm;
    //
    // again change sign w.r.t. g4
    //
    rm.rotateY(gmt_mgr->PixelModuleAngle()*gmt_mgr->PixelModuleAngleSign(ii) );
//
// Place the Module
//
    GeoVPhysVol* modulephys = pm.Build() ;
    GeoNameTag *tag = new GeoNameTag("Module");         
    GeoAlignableTransform* xform;
    // OLD EXAMPLE FOR ALIGNEMENT!!
    //    if(gmt_mgr->IsAlign() ) {
    //      xform = new GeoSiAlTransform(HepTransform3D(rm,modulepos), pm.getID() );
    //    } else {
    //      xform = new GeoTransform(HepTransform3D(rm,modulepos));
    //    }
    xform = new GeoAlignableTransform(HepTransform3D(rm,modulepos));
    ladderPhys->add(tag);
    ladderPhys->add(new GeoIdentifierTag(gmt_mgr->Eta() ) );
    ladderPhys->add(xform);
    ladderPhys->add(modulephys );

    // Now store the xform by identifier:
    Identifier id = m_theSensor.getID();
    DDmgr->addAlignableTransform(0,id,xform,modulephys);
    
  }

  //
  // Place the TMT
  //
  gmt_mgr->SetCurrentTrapType("TMT");
  GeoPixelTrapTmt trap;
  for (int ii = 0; ii < gmt_mgr->PixelNTrapC()+gmt_mgr->PixelNTrapE(); ii++) {
    gmt_mgr->SetCurrentTrap(ii);
    GeoVPhysVol* trapphys = trap.Build() ;
    double xpos = gmt_mgr->PixelTrapPosx();
    double ypos = gmt_mgr->PixelTrapPosy();
    double zpos = gmt_mgr->PixelTrapPosz();
    //std::cout << "placing central or ending trap "<<gmt_mgr->PixelTrapLabel()<<" at position "<< xpos<<" "<< ypos<<" "<<zpos <<std::endl;
    Hep3Vector trappos(xpos,ypos,zpos);
    GeoTransform* xform;
    xform = new GeoTransform(HepTransform3D(HepRotation(),trappos));
    GeoNameTag *tag = new GeoNameTag("TrapCE");  
    ladderPhys->add(tag);
    ladderPhys->add(new GeoIdentifierTag(ii) );
    ladderPhys->add(xform);
    ladderPhys->add(trapphys );  
  } 
  HepRotation traprot;
  traprot.rotateX(180.*deg);
  //JBdV Test
  for (int ii = 0; ii < HalfNModule; ii++) {
  //for (int ii = 0; ii < 1; ii++) {
    double zshift = gmt_mgr->PixelModulePosition(1)*ii;
    for (int jj = 0; jj < gmt_mgr->PixelNTrap(); jj++) {
      gmt_mgr->SetCurrentTrap(jj+gmt_mgr->PixelNTrapC()+gmt_mgr->PixelNTrapE());
      GeoVPhysVol* trapphys = trap.Build() ;
      double xpos = gmt_mgr->PixelTrapPosx();
      double ypos = gmt_mgr->PixelTrapPosy();
      double zpos = gmt_mgr->PixelTrapPosz()+zshift;
      //std::cout << "placing trap "<<gmt_mgr->PixelTrapLabel()<<" at position "<< xpos<<" "<< ypos<<" "<<zpos <<std::endl;
      Hep3Vector trappos(xpos,ypos,zpos);
      GeoTransform* xform;
      xform = new GeoTransform(HepTransform3D(HepRotation(),trappos));
      GeoNameTag *tag = new GeoNameTag("Trap");  
      ladderPhys->add(tag);
      ladderPhys->add(new GeoIdentifierTag(ii) );
      ladderPhys->add(xform);
      ladderPhys->add(trapphys);
      //
      GeoVPhysVol* trapphysop = trap.Build() ;
      zpos = -zpos;
      Hep3Vector trapposop(xpos,ypos,zpos);
      GeoTransform* xformop;
      xformop = new GeoTransform(HepTransform3D(traprot,trapposop));
      GeoNameTag *tagop = new GeoNameTag("Trap");  
      ladderPhys->add(tagop);
      ladderPhys->add(new GeoIdentifierTag(ii) );
      ladderPhys->add(xformop);
      ladderPhys->add(trapphysop);
    }
  } 

//   //
//   // Place Pigtail + connector
//   //
//   double thickness = gmt_mgr->PixelLadderThickness() + gmt_mgr->PixelCableThickness() + 0.25*cm; // plus 0.25 cm ???
//   double yshift    = (gmt_mgr->PixelTrapWidth() - gmt_mgr->PixelLadderWidth())/2. 
//     - (this->Thickness()/2. + thickness/2.)*tan(gmt_mgr->PixelLadderTilt());
//   GeoPixelPosi posi;
//   for (int ii = 0; ii<gmt_mgr->PixelNModule(); ii++) {
//     int indexz    = abs(ii-6);
//     double zshift = gmt_mgr->PixelModulePosition(indexz);
//     if (ii < 6) zshift = -zshift;
//     for (int jj = 0; jj < gmt_mgr->PixelConnectorANParaEl(); jj++) {
//       gmt_mgr->SetCurrentParaEl(jj);
//       gmt_mgr->SetCurrentParaElType("Poai");
//       GeoVPhysVol* poaiphys = posi.Build();
//       double xpos = gmt_mgr->PixelPosiPosx() - thickness/2.;
//       double ypos = gmt_mgr->PixelPosiPosy() + yshift;
//       double zpos = gmt_mgr->PixelPosiPosz() + zshift;
//       Hep3Vector poaipos(xpos,ypos,zpos);
//       GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),poaipos));
//       GeoNameTag *tag = new GeoNameTag("ConnecParaA");  
//       ladderPhys->add(tag);
//       ladderPhys->add(new GeoIdentifierTag(ii));
//       ladderPhys->add(xform);
//       ladderPhys->add(poaiphys);
//     }
//     for (int jj = 0; jj < gmt_mgr->PixelConnectorBNCyliEl(); jj++) {
//       gmt_mgr->SetCurrentCyliEl(jj);
//       double rmin   = gmt_mgr->PixelPotiRmin("Pobi");
//       double rmax   = gmt_mgr->PixelPotiRmax("Pobi");
//       double length = gmt_mgr->PixelPotiHalfLength("Pobi");
//       double phi1   = gmt_mgr->PixelPotiPhi1("Pobi");
//       double dphi   = gmt_mgr->PixelPotiPhi2("Pobi")-phi1;
//       std::string logName = "CyliConnecB";
//       std::ostringstream o;
//       o << gmt_mgr->PixelPotiLabel("Pobi");
//       logName = logName+o.str();
//       GeoMaterial* cyliMat = mat_mgr->getMaterial("pix::Ladder");
//       const GeoTubs* tubs  = new GeoTubs(rmin,rmax,length,phi1,dphi);
//       GeoLogVol* theTubs   = new GeoLogVol(logName,tubs,cyliMat);
//       GeoPhysVol* pobiphys = new GeoPhysVol(theTubs);
//       GeoNameTag* tag      = new GeoNameTag("ConnecCyli"); 
//       double xpos = gmt_mgr->PixelPotiPosx("Pobi") - thickness/2.;
//       double ypos = gmt_mgr->PixelPotiPosy("Pobi") + yshift;
//       double zpos = gmt_mgr->PixelPotiPosz("Pobi") + zshift;
//       Hep3Vector pobipos(xpos,ypos,zpos);
//       GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pobipos));
//       ladderPhys->add(tag);
//       ladderPhys->add(new GeoIdentifierTag(ii));
//       ladderPhys->add(xform);
//       ladderPhys->add(pobiphys);
//     }
//   }
//   for (int ii = 0; ii<gmt_mgr->PixelConnectorCNParaEl(); ii++) {
//     gmt_mgr->SetCurrentParaEl(ii);
//     gmt_mgr->SetCurrentParaElType("Poci");
//     GeoVPhysVol* pociphys = posi.Build();
//     double xpos = gmt_mgr->PixelPosiPosx();
//     double ypos = gmt_mgr->PixelPosiPosy();
//     double zpos = gmt_mgr->PixelPosiPosz();
//     Hep3Vector pocipos(xpos,ypos,zpos);
//     GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pocipos));
//     GeoNameTag *tag = new GeoNameTag("ConnecParaC");  
//     ladderPhys->add(tag);
//     ladderPhys->add(new GeoIdentifierTag(ii));
//     ladderPhys->add(xform);
//     ladderPhys->add(pociphys);
//   }

  return ladderPhys;
}


double GeoPixelLadder::Thickness() {
  //
  // The thickness of the ladderis calculated starting from the thickness of
  // its components: Board + Hybrid+Chips
  // then the distance of the modules from the center is added and
  // a factor is added to account for the tilt of ~1degree given to
  // have a z overlap of the single modules
  //
  double tckincl = (gmt_mgr->PixelBoardLength()+gmt_mgr->PixelBalcony())*
    sin(gmt_mgr->PixelModuleAngle())/2.;
//   double thick = gmt_mgr->PixelBoardThickness()+
//     gmt_mgr->PixelHybridThickness()+
//     gmt_mgr->PixelChipThickness()+
//     fabs(gmt_mgr->PixelModuleDrDistance())+
//     tckincl;
//JBdV new DC3
  double thick = 2.*tckincl + gmt_mgr->PixelBoardThickness() +
    gmt_mgr->PixelChipThickness() + 2.*gmt_mgr->PixelGapDetLad();
  return thick;
}
