#include "PixelGeoModel/GeoPixelModule.h"
#include "PixelGeoModel/GeoPixelHybrid.h"
#include "PixelGeoModel/GeoPixelChip.h"
#include "PixelGeoModel/GeoPixelSiCrystal.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoFullPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoTransform.h"

using std::max;

GeoPixelModule::GeoPixelModule(GeoPixelSiCrystal& theSensor) :
  m_theSensor(theSensor)
{
  //
  // Define the log volume in the constructor, so I do it only once.
  //
  //
  // The Dimensions are in separate routines
  //
  double length = this->Length();
  double thickness = this->Thickness();
  double width = this->Width();
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  const GeoBox* moduleBox = new GeoBox(thickness/2.,width/2.,length/2.);
  theModule = new GeoLogVol("moduleLog",moduleBox,air);
}

GeoVPhysVol* GeoPixelModule::Build( ) {
  GeoFullPhysVol* modulePhys = new GeoFullPhysVol(theModule);
  //
  // Place the Si Crystal
  //
  GeoVPhysVol *theSi = m_theSensor.Build();
  _id = m_theSensor.getID();
  GeoNameTag *tag = new GeoNameTag("Si Crystal");         
  GeoTransform *xformsi = new GeoTransform(HepTransform3D());
  modulePhys->add(tag);
  modulePhys->add(new GeoIdentifierTag(100) );
  modulePhys->add(xformsi);
  modulePhys->add(theSi );
  //
  // Place the Hybrid
  //
  GeoPixelHybrid ph;
  double xpos = -0.5*(gmt_mgr->PixelBoardThickness()+gmt_mgr->PixelHybridThickness());
  Hep3Vector hybpos(xpos,0.,0.);
  GeoTransform* xform = new GeoTransform(HepTransform3D( HepRotation(),hybpos) );
  tag = new GeoNameTag("Hybrid");
  modulePhys->add(tag);
  modulePhys->add(xform);
  modulePhys->add(ph.Build() );
  //
  // Place the Chip
  //
  GeoPixelChip pc;
  xpos = 0.5*(gmt_mgr->PixelBoardThickness()+gmt_mgr->PixelChipThickness())+gmt_mgr->PixelChipGap();
  Hep3Vector chippos(xpos,0.,0.);
  xform = new GeoTransform(HepTransform3D (HepRotation(),chippos) );
  tag = new GeoNameTag("Chip");
  modulePhys->add(tag);
  modulePhys->add(xform);
  modulePhys->add(pc.Build() );
  //
  // Add the silicon element to the list of detector elements...
  //
  return modulePhys;
}


double GeoPixelModule::Thickness() {
  //
  // The thickness of the module is given by the board thickness
  // plus the maximum b/w the hybrid and the chip+gap
  //
  double thick = gmt_mgr->PixelBoardThickness()+
    2*max(gmt_mgr->PixelHybridThickness(),
        gmt_mgr->PixelChipThickness()+gmt_mgr->PixelChipGap());
  //
  // Don't understand why....
  //
  return thick+.01;
}

double GeoPixelModule::Width() {
  double width = max( max(
                     gmt_mgr->PixelBoardWidth(),
                     gmt_mgr->PixelHybridWidth()),
                     gmt_mgr->PixelChipWidth());
  return width;
}
double GeoPixelModule::Length() {
  double length = max( max( max(
                                gmt_mgr->PixelHybridLength(),
                                gmt_mgr->PixelBoardLength()),
                            2*gmt_mgr->PixelBalcony()),
                       gmt_mgr->PixelChipLength());

  return length;
}

Identifier GeoPixelModule::getID() {
  return _id;
}
