#include "PixelGeoModel/GeoPixelTubeCables.h"
#include "PixelGeoModel/GeoPixelLadderStruct.h"
#include "PixelGeoModel/GeoPixelCable.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
// #include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoTransform.h"

GeoPixelTubeCables::GeoPixelTubeCables() {
  //
  // Define the log volume in the constructor, so I do it only once.
  //

  //   
  // Length is in the db
  //
  double halflength = gmt_mgr->PixelLadderHalfLength();
  //
  // The width is the maximum among the componenst widths
  //
  double width = std::max(gmt_mgr->PixelLadderWidth(),
                     gmt_mgr->PixelCableWidth());
  //
  // The thickness has to be calculated
  //
  double thickness = this->Thickness();
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  const GeoBox* solBox = new GeoBox(thickness/2.,width/2.,halflength+m_epsilon);
  theBox = new GeoLogVol("TubeCablesLog",solBox,air);
}


GeoVPhysVol* GeoPixelTubeCables::Build( ) {
  GeoPhysVol* TCPhys = new GeoPhysVol(theBox);
  //
  // Place the Ladder Structure. Don't understand why this is going in the
  // tube & cables section...
  //
  GeoPixelLadderStruct pls;
  GeoNameTag* tag = new GeoNameTag("LadderStructure");
  GeoVPhysVol* ladderstructPhys =  pls.Build() ;
  double xpos = 0.5*(-this->Thickness()+gmt_mgr->PixelLadderThickness());
  Hep3Vector pos(xpos,0.,0.);
  GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pos));
  TCPhys->add(tag);
  TCPhys->add(xform);
  TCPhys->add(ladderstructPhys);
  //
  // Place the cables... ouch!
  //
  // I do it in a symmetric way, assuming that the routing of the
  // central module is done half on the right and half on the left.
  // thus I have 7 cables running on each side.
  //
  GeoPixelCable pc;
  double xcabshift = 0.;
  for(int ii = 0; ii <= gmt_mgr->PixelNModule()/2; ii++) {
    pc.SetModuleNumber(ii);
    double zcabpos = gmt_mgr->PixelCableZMax()-pc.Length()/2.+m_epsilon/2.;
    xcabshift += pc.Thickness()/2.;
    double xcabpos = - this->Thickness()/2. + gmt_mgr->PixelLadderThickness()+xcabshift;
    //
    // Place the cables on both sides.
    // piling them up. this is slightly different from what in G3
    // where the cables are not piled up, I don't see a good reason for
    // that. In G3 there is an empty space b/w the cable from the center
    // and the cable from the first module, as the shift is not calculated
    // in the same way.
    //
    xcabshift += pc.Thickness()/2.;
    Hep3Vector cablepos(xcabpos,0.,zcabpos);
    GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),cablepos));
    GeoNameTag *tag = new GeoNameTag("Cable");         
    //
    // Left side
    //
    GeoVPhysVol *cablePhys1 =  pc.Build();
    TCPhys->add(tag);
    TCPhys->add(xform);
    //    TCPhys->add(new GeoIdentifierTag(ii) );
    TCPhys->add(cablePhys1);
    //
    // Right side
    //
    GeoVPhysVol *cablePhys2 =  pc.Build();
    cablepos = Hep3Vector(xcabpos,0.,-zcabpos);
    xform = new GeoTransform(HepTransform3D(HepRotation(),cablepos));
    TCPhys->add(tag);
    TCPhys->add(xform);
    //    TCPhys->add(new GeoIdentifierTag(ii+100) );
    TCPhys->add(cablePhys2);
  }

  return TCPhys;
}


double GeoPixelTubeCables::Thickness() {
  //
  // The thickness of the box is calculated starting from the thickness of
  // its components: Ladder + cables
  //
  double thick = gmt_mgr->PixelLadderThickness()+
    gmt_mgr->PixelCableThickness();
  return thick;
}




