//
// This class builds the Omega (+ Tube Al)
//
// The information on which layer to build is retrieved by the Geometry Manager
//
//
// Contains: nothing
//
#include "PixelGeoModel/GeoPixelTubeOmega.h"
#include "PixelGeoModel/GeoPixelPosi.h"
#include "PixelGeoModel/GeoPixelTrapTmt.h"
#include "GeoModelKernel/GeoTubs.h"
#include "GeoModelKernel/GeoBox.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoNameTag.h"
#include "GeoModelKernel/GeoIdentifierTag.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"
#include "GeoModelKernel/GeoTransform.h"

#include<math.h>
#include <sstream>

GeoPixelTubeOmega::GeoPixelTubeOmega() {
  //
  // Define the log volume in the constructor, so I do it only once.
  // m_myepsilon so that the para elements have no common faces with the
  // box (otherwise, G4 thinks that they are not inside the mother)
  //   
  // Length and thickness of the Omega are in the db
  //
  gmt_mgr->SetCurrentParaElType("Posi");
  gmt_mgr->SetCurrentParaEl(9);
  double halflength = gmt_mgr->PixelPosiHalfLength()+m_epsilon;
  //double thickness  = gmt_mgr->PixelLadderThickness()+gmt_mgr->PixelCableThickness()+m_epsilon;
  double thickness  = gmt_mgr->PixelLadderThickness()+gmt_mgr->PixelCableThickness()+ 2.5 + m_epsilon;// plus 0.25 cm New DC3 ???
  double width      = gmt_mgr->PixelLadderWidth()+m_epsilon;
  //
  // from Nova
  GeoMaterial* air = mat_mgr->getMaterial("std::Air");
  //
  const GeoBox* omegaBox = new GeoBox(thickness/2.,width/2.,halflength);
  theOmega = new GeoLogVol("omegaLog",omegaBox,air);
}


GeoVPhysVol* GeoPixelTubeOmega::Build() {
  GeoPhysVol* omegaPhys = new GeoPhysVol(theOmega);
  //double thickness = gmt_mgr->PixelLadderThickness()+gmt_mgr->PixelCableThickness();
  double thickness = gmt_mgr->PixelLadderThickness() + gmt_mgr->PixelCableThickness() + 0.25*cm; // plus 0.25 cm New DC3 ???
  //
  // The para parts
  //
  GeoPixelPosi para;
  for (int ii = 0;ii<gmt_mgr->PixelOmegaNParaEl();ii++) {
    gmt_mgr->SetCurrentParaEl(ii);
    gmt_mgr->SetCurrentParaElType("Posi");
    GeoVPhysVol* paraPhys = para.Build() ;
    GeoNameTag *tag = new GeoNameTag("ParalElOmega"); 
    // add to the Omega
    float xpos = gmt_mgr->PixelPosiPosx()-thickness/2.;
    float ypos = gmt_mgr->PixelPosiPosy();
    float zpos = gmt_mgr->PixelPosiPosz();
    Hep3Vector posipos(xpos,ypos,zpos);
    HepRotation rm;
    rm.rotateZ(gmt_mgr->PixelPosiRotz());
    GeoTransform* xform = new GeoTransform(HepTransform3D(rm,posipos));
    omegaPhys->add(tag);
    omegaPhys->add(new GeoIdentifierTag(ii));
    omegaPhys->add(xform);
    omegaPhys->add(paraPhys);
  }  
  //
  // The tubs parts
  //
  for (int ii = 0;ii<gmt_mgr->PixelOmegaNCyliEl();ii++) {
    gmt_mgr->SetCurrentCyliEl(ii);
    int imat      = gmt_mgr->PixelMaterialCyli("Poti");
    double rmin   = gmt_mgr->PixelPotiRmin("Poti");
    double rmax   = gmt_mgr->PixelPotiRmax("Poti");
    if (rmin != rmax) {
      double length = gmt_mgr->PixelPotiHalfLength("Poti");
      double phi1   = gmt_mgr->PixelPotiPhi1("Poti");
      double dphi   = gmt_mgr->PixelPotiPhi2("Poti")-phi1;
      std::string logName ="CyliTub";
      std::ostringstream o;
      o << gmt_mgr->PixelPotiLabel("Poti");
      logName = logName+o.str();
      GeoMaterial* cyliMat = mat_mgr->getMaterial(gmt_mgr->PixelGetMat(imat));
      const GeoTubs* tubs  = new GeoTubs(rmin,rmax,length,phi1,dphi);
      GeoLogVol* theTubs   = new GeoLogVol(logName,tubs,cyliMat);
      GeoPhysVol* tubsPhys = new GeoPhysVol(theTubs);
      GeoNameTag *tag = new GeoNameTag("CylinElOmega"); 
      // add to the Omega
      float xpos = gmt_mgr->PixelPotiPosx("Poti")-thickness/2.;
      float ypos = gmt_mgr->PixelPotiPosy("Poti");
      float zpos = gmt_mgr->PixelPotiPosz("Poti");
      Hep3Vector potipos(xpos,ypos,zpos);
      GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),potipos));
      omegaPhys->add(tag);
      omegaPhys->add(new GeoIdentifierTag(ii));
      omegaPhys->add(xform);
      omegaPhys->add(tubsPhys); 
      // Add the weight (cheks)
      double weicyli = tubs->volume()*cyliMat->getDensity()/gram;
      //std::cout<<"And the "<<gmt_mgr->PixelPotiLabel("Poti") <<"th cyli (poti) Weight is "<<weicyli<<" grams"<<std::endl;
      gmt_mgr->AddWeight(weicyli);
      gmt_mgr->AddWeightOmega(weicyli);
    }
  }
  //
  // Put the fluid/gaz mixture
  //
  gmt_mgr->SetCurrentTrapType("Fluid");
  int LadType = gmt_mgr->PixelFluidOrient();
  int Capilar = LadType%2 + 1; if (Capilar == 2) Capilar = 0;
  gmt_mgr->SetCurrentTrap(Capilar);
  GeoPixelTrapTmt trap;
  HepRotation rm;
  if (LadType > 2) {
    rm.rotateX(180.);
  }
  double xpos = gmt_mgr->PixelTrapPosx()-thickness/2.;
  double ypos = gmt_mgr->PixelTrapPosy();
  double zpos = gmt_mgr->PixelTrapPosz();
  Hep3Vector fluidpos(xpos,ypos,zpos);
  GeoVPhysVol* trapPhys = trap.Build() ;
  GeoNameTag *tag       = new GeoNameTag("FluidTrap"); 
  GeoTransform* xform   = new GeoTransform(HepTransform3D(rm,fluidpos));
  omegaPhys->add(tag);
  omegaPhys->add(new GeoIdentifierTag(LadType));
  omegaPhys->add(xform);
  omegaPhys->add(trapPhys);
  //
  // Place Pigtail + connector
  //
  //double yshift    = (gmt_mgr->PixelTrapWidth() - gmt_mgr->PixelLadderWidth())/2. 
  // - (this->Thickness()/2. + thickness/2.)*tan(fabs(gmt_mgr->PixelLadderTilt()));
  double yshift = 0.;
  //std::cout << "yshift = " << yshift << std::endl;
  GeoPixelPosi posi;
  double yposa = 0;
  for (int ii = 0; ii<gmt_mgr->PixelNModule(); ii++) {
    int indexz    = abs(ii-6);
    double zshift = gmt_mgr->PixelModulePosition(indexz);
    if (ii < 6) zshift = -zshift;
    for (int jj = 0; jj < gmt_mgr->PixelConnectorANParaEl(); jj++) {
      gmt_mgr->SetCurrentParaEl(jj);
      gmt_mgr->SetCurrentParaElType("Poai");
      GeoVPhysVol* poaiphys = posi.Build();
      double xpos = gmt_mgr->PixelPosiPosx() - thickness - this->Thickness()/2.;
      //Test double ypos = gmt_mgr->PixelPosiPosy() + yshift;
      double ypos = gmt_mgr->PixelLadderWidth()/2. - gmt_mgr->PixelPosiWidth()/2.;
      double zpos = gmt_mgr->PixelPosiPosz() + zshift;
//       std::cout << "Placing Poai "<<gmt_mgr->PixelPosiLabel() << " at x = " << xpos << "("
// 		<< gmt_mgr->PixelPosiPosx() << " "<< thickness << " " <<this->Thickness()/2.<< ")" <<std::endl;
      Hep3Vector poaipos(xpos,ypos,zpos);
      GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),poaipos));
      GeoNameTag *tag = new GeoNameTag("ConnecParaA");  
      omegaPhys->add(tag);
      omegaPhys->add(new GeoIdentifierTag(ii));
      omegaPhys->add(xform);
      omegaPhys->add(poaiphys);
      yposa = ypos;
    }
//     for (int jj = 0; jj < gmt_mgr->PixelConnectorBNCyliEl(); jj++) {
//       gmt_mgr->SetCurrentCyliEl(jj);
//       int imat      = gmt_mgr->PixelMaterialCyli("Pobi");
//       double rmin   = gmt_mgr->PixelPotiRmin("Pobi");
//       double rmax   = gmt_mgr->PixelPotiRmax("Pobi");
//       double length = gmt_mgr->PixelPotiHalfLength("Pobi");
//       double phi1   = gmt_mgr->PixelPotiPhi1("Pobi");
//       double dphi   = gmt_mgr->PixelPotiPhi2("Pobi")-phi1;
//       std::string logName = "CyliConnecB";
//       std::ostringstream o;
//       o << gmt_mgr->PixelPotiLabel("Pobi");
//       logName = logName+o.str();
//       GeoMaterial* cyliMat = mat_mgr->getMaterial(gmt_mgr->PixelGetMat(imat));
//       const GeoTubs* tubs  = new GeoTubs(rmin,rmax,length,phi1,dphi);
//       GeoLogVol* theTubs   = new GeoLogVol(logName,tubs,cyliMat);
//       GeoPhysVol* pobiphys = new GeoPhysVol(theTubs);
//       GeoNameTag* tag      = new GeoNameTag("ConnecCyli"); 
//       double xpos = gmt_mgr->PixelPotiPosx("Pobi") - thickness - 1.*mm; // Test;
//       double ypos = gmt_mgr->PixelPotiPosy("Pobi") + yshift - 3.*mm; // Test
//       double zpos = gmt_mgr->PixelPotiPosz("Pobi") + zshift;
//       Hep3Vector pobipos(xpos,ypos,zpos);
//       GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pobipos));
//       omegaPhys->add(tag);
//       omegaPhys->add(new GeoIdentifierTag(ii));
//       omegaPhys->add(xform);
//       omegaPhys->add(pobiphys);
//       // Add the weight (cheks)
//       double weicyli = tubs->volume()*cyliMat->getDensity()/gram;
//       //std::cout<<"And the "<<gmt_mgr->PixelPotiLabel("Pobi") <<"th cyli (pobi) Weight is "<<weicyli<<" grams"<<std::endl;
//       gmt_mgr->AddWeight(weicyli);
//       gmt_mgr->AddWeightCable(weicyli);
//     }
  }
  for (int ii = 0; ii<gmt_mgr->PixelConnectorCNParaEl(); ii++) {
    gmt_mgr->SetCurrentParaEl(ii);
    gmt_mgr->SetCurrentParaElType("Poci");
    GeoVPhysVol* pociphys = posi.Build();
    double xpos = gmt_mgr->PixelPosiPosx() - thickness/2. - this->Thickness()/2.;
    //Test double ypos = gmt_mgr->PixelPosiPosy();
    double ypos = yposa;
    double zpos = gmt_mgr->PixelPosiPosz();
    if (ii == 7) zpos += 0.000001;
    //std::cout << "Placing Poci "<<gmt_mgr->PixelPosiLabel() << " at x = " << xpos << " z = "<< zpos << std::endl;
    Hep3Vector pocipos(xpos,ypos,zpos);
    GeoTransform* xform = new GeoTransform(HepTransform3D(HepRotation(),pocipos));
    GeoNameTag *tag = new GeoNameTag("ConnecParaC");  
    omegaPhys->add(tag);
    omegaPhys->add(new GeoIdentifierTag(ii));
    omegaPhys->add(xform);
    omegaPhys->add(pociphys);
  }

  return omegaPhys;
}

double GeoPixelTubeOmega::Thickness() {
  double tckincl = (gmt_mgr->PixelBoardLength()+gmt_mgr->PixelBalcony())*
    sin(gmt_mgr->PixelModuleAngle())/2.;
  double thick = 2.*tckincl + gmt_mgr->PixelBoardThickness() +
    gmt_mgr->PixelChipThickness() + 2.*gmt_mgr->PixelGapDetLad();
  return thick;
}
